@@ -5,17 +5,21 @@ Documentation for soxspipe can be found here: http://soxspipe.readthedocs.org
 
 Usage:
     soxspipe init
-    soxspipe mbias <inputFrames> [-s <pathToSettingsFile>] 
-    soxspipe mdark <inputFrames> [-s <pathToSettingsFile>] 
-    soxspipe disp_sol <inputFrames> [-s <pathToSettingsFile>] 
-    soxspipe order_centres <inputFrames> [-s <pathToSettingsFile>] 
+    soxspipe mbias <inputFrames> [-o <outputDirectory> -s <pathToSettingsFile>] 
+    soxspipe mdark <inputFrames> [-o <outputDirectory> -s <pathToSettingsFile>]
+    soxspipe mflat <inputFrames> [-o <outputDirectory> -s <pathToSettingsFile>]
+    soxspipe disp_sol <inputFrames> [-o <outputDirectory> -s <pathToSettingsFile>]
+    soxspipe order_centres <inputFrames> [-o <outputDirectory> -s <pathToSettingsFile>]
+    soxspipe spat_sol <inputFrames> [-o <outputDirectory> -s <pathToSettingsFile>]
 
 Options:
     init                                   setup the soxspipe settings file for the first time
     mbias                                  the master bias recipe
     mdark                                  the master dark recipe
+    mflat                                  the master flat recipe
     disp_sol                               the disp solution recipe
     order_centres                          the order centres recipe
+    spat_sol                               the spatial solution recipe
 
     inputFrames                            path to a directory of frames or a set-of-files file
 
@@ -114,6 +118,10 @@ def main(arguments=None):
             pickleMe[k] = theseLocals[k]
         pickle.dump(pickleMe, open(pathToPickleFile, "wb"))
 
+    # PACK UP SOME OF THE CL SWITCHES INTO SETTINGS DICTIONARY
+    if a['outputDirectory']:
+        settings["intermediate-data-root"] = a['outputDirectory']
+
     if a["init"]:
         from os.path import expanduser
         home = expanduser("~")
@@ -168,6 +176,25 @@ def main(arguments=None):
         ).produce_product()
         print(f"\nThe order centre locations have been saved to an order table: {order_table}")
 
+    if a["spat_sol"]:
+        from soxspipe.recipes import soxs_spatial_solution
+        disp_map = soxs_spatial_solution(
+            log=log,
+            settings=settings,
+            inputFrames=a["inputFrames"]
+        ).produce_product()
+        print(f"\nFull 2D dispersion map saved to: {disp_map}")
+
+    if a["mflat"]:
+        from soxspipe.recipes import soxs_mflat
+        recipe = soxs_mflat(
+            log=log,
+            settings=settings,
+            inputFrames=a["inputFrames"]
+        )
+        mflatFrame = recipe.produce_product()
+        print(f"\nThe master flat frame has been saved to: {mflatFrame}")
+
     # CALL FUNCTIONS/OBJECTS
 
     if "dbConn" in locals() and dbConn:
