@@ -32,6 +32,10 @@ from photutils.utils import NoDetectionsWarning
 import warnings
 from astropy.visualization import hist
 from soxspipe.commonutils.polynomials import chebyshev_order_wavelength_polynomials
+from soxspipe.commonutils.filenamer import filenamer
+from soxspipe.commonutils.dispersion_map_to_pixel_arrays import dispersion_map_to_pixel_arrays
+import pandas as pd
+from tabulate import tabulate
 
 
 class create_dispersion_map(object):
@@ -42,36 +46,33 @@ class create_dispersion_map(object):
         - ``log`` -- logger
         - ``settings`` -- the settings dictionary
         - ``pinholeFrame`` -- the calibrated pinhole frame (single or multi)
+        - ``firstGuessMap`` -- the first guess dispersion map from the `soxs_disp_solution` recipe (needed in `soxs_spat_solution` recipe). Default *False*.
 
     **Usage:**
 
-    ```eval_rst
-    .. todo::
-
-        - add a tutorial about ``create_dispersion_map`` to documentation
-    ```
-
     ```python
     from soxspipe.commonutils import create_dispersion_map
     mapPath = create_dispersion_map(
         log=log,
         settings=settings,
-        pinholeFrame=frame
+        pinholeFrame=frame,
+        firstGuessMap=False
     ).get()
     ```
-
     """
 
     def __init__(
             self,
             log,
             settings,
-            pinholeFrame
+            pinholeFrame,
+            firstGuessMap=False
     ):
         self.log = log
-        log.debug("instansiating a new 'create_dispersion_map' object")
+        log.debug("instantiating a new 'create_dispersion_map' object")
         self.settings = settings
         self.pinholeFrame = pinholeFrame
+        self.firstGuessMap = firstGuessMap
 
         # KEYWORD LOOKUP OBJECT - LOOKUP KEYWORD FROM DICTIONARY IN RESOURCES
         # FOLDER
@@ -98,66 +99,58 @@ class create_dispersion_map(object):
 
         **Return:**
             - ``mapPath`` -- path to the file containing the coefficients of the x,y polynomials of the global dispersion map fit
-
-        **Usage:**
-
-        ```eval_rst
-        .. todo::
-
-            - update the package tutorial if needed
-        ```
         """
         self.log.debug('starting the ``get`` method')
 
-        # READ PREDICTED LINE POSITIONS FROM FILE
-        predictedLines = self.get_predicted_line_list()
+        # WHICH RECIPE ARE WE WORKING WITH?
+        if self.firstGuessMap:
+            recipe = "soxs-spatial-solution"
+            slit_deg = self.settings[recipe]["slit-deg"]
+        else:
+            recipe = "soxs-disp-solution"
+            slit_deg = 0
 
-        # ROUND OFF THE PIXEL POSITIONS
-        observed_x = []
-        observed_y = []
-        windowSize = self.settings["soxs-disp-solution"]["pixel-window-size"]
-        windowHalf = int(windowSize / 2)
+        # READ PREDICTED LINE POSITIONS FROM FILE - RETURNED AS DATAFRAME
+        orderPixelTable = self.get_predicted_line_list()
 
-        # DETECT THE LINES ON THE PINHILE FRAME
-        for x, y in zip(predictedLines["detector_x"].astype(int), predictedLines["detector_y"].astype(int)):
-            found_x, found_y = self.detect_pinhole_arc_line(x, y, windowHalf)
-            observed_x.append(found_x)
-            observed_y.append(found_y)
+        # GET THE WINDOW SIZE FOR ATTEMPTING TO DETECT LINES ON FRAME
+        windowSize = self.settings[recipe]["pixel-window-size"]
+        self.windowHalf = int(windowSize / 2)
 
-        # COLLECT INFO FOR DETECTED LINES
-        wavelengths = [w for w, x in zip(
-            predictedLines["Wavelength"], observed_x) if x != None]
-        orders = [w for w, x in zip(
-            predictedLines["Order"], observed_x) if x != None]
-        observed_x = [x for x in observed_x if x != None]
-        observed_y = [y for y in observed_y if y != None]
+        # DETECT THE LINES ON THE PINHILE FRAME AND
+        # ADD OBSERVED LINES TO DATAFRAME
+        orderPixelTable = orderPixelTable.apply(
+            self.detect_pinhole_arc_line, axis=1)
 
-        order_deg = self.settings["soxs-disp-solution"]["order-deg"]
-        wavelength_deg = self.settings["soxs-disp-solution"]["wavelength-deg"]
+        # DROP MISSING VALUES
+        orderPixelTable.dropna(axis='index', how='any', subset=[
+            'observed_x'], inplace=True)
+
+        order_deg = self.settings[recipe]["order-deg"]
+        wavelength_deg = self.settings[
+            recipe]["wavelength-deg"]
 
         # ITERATIVELY FIT THE POLYNOMIAL SOLUTIONS TO THE DATA
         popt_x, popt_y = self.fit_polynomials(
-            observed_x=observed_x,
-            observed_y=observed_y,
-            wavelengths=wavelengths,
-            orders=orders,
+            orderPixelTable=orderPixelTable,
+            wavelength_deg=wavelength_deg,
             order_deg=order_deg,
-            wavelength_deg=wavelength_deg
+            slit_deg=slit_deg,
         )
 
         # WRITE THE MAP TO FILE
         mapPath = self.write_map_to_file(
-            popt_x, popt_y, order_deg, wavelength_deg)
+            popt_x, popt_y, order_deg, wavelength_deg, slit_deg)
 
         self.log.debug('completed the ``get`` method')
         return mapPath
 
     def get_predicted_line_list(
             self):
-        """*lift the predicted line list from the static calcibrations*
+        """*lift the predicted line list from the static calibrations*
 
         **Return:**
-            - ``predictedLines`` -- a dictionary of lists detailing Wavelength,Order,slit_index,slit_position,detector_x,detector_y
+            - ``orderPixelTable`` -- a panda's data-frame containing wavelength,order,slit_index,slit_position,detector_x,detector_y
         """
         self.log.debug('starting the ``get_predicted_line_list`` method')
 
@@ -188,31 +181,81 @@ class create_dispersion_map(object):
         calibrationRootPath = self.settings[
             "calibration-data-root"].replace("~", home)
         predictedLinesFile = calibrationRootPath + "/" + dp["predicted pinhole lines"][frameTech][f"{binx}x{biny}"]
-        predictedLines = np.genfromtxt(
-            predictedLinesFile, delimiter=',', names=True)
+
+        # READ CSV FILE TO PANDAS DATAFRAME
+        orderPixelTable = pd.read_csv(predictedLinesFile)
+
+        # RENAME ALL COLUMNS FOR CONSISTENCY
+        listName = []
+        listName[:] = [l if l else l for l in listName]
+        orderPixelTable.columns = [d.lower() if d.lower() in [
+            "order", "wavelength"] else d for d in orderPixelTable.columns]
+
+        # WANT TO DETERMINE SYSTEMATIC SHIFT IF FIRST GUESS SOLUTION PRESENT
+        if self.firstGuessMap:
+            # ADD SOME EXTRA COLUMNS TO DATAFRAME
+
+            # FILTER THE PREDICTED LINES TO ONLY SLIT POSITION INCLUDED IN
+            # SINGLE PINHOLE FRAMES
+            slitIndex = int(dp["mid_slit_index"])
+
+            # GET THE OBSERVED PIXELS VALUES
+            orderPixelTable = dispersion_map_to_pixel_arrays(
+                log=self.log,
+                dispersionMapPath=self.firstGuessMap,
+                orderPixelTable=orderPixelTable
+            )
+
+            # CREATE A COPY OF THE DATA-FRAME TO DETERMINE SHIFTS
+            tmpList = orderPixelTable.copy()
+
+            mask = (tmpList['slit_index'] == slitIndex)
+            tmpList.loc[mask, 'shift_x'] = tmpList.loc[
+                mask, 'detector_x'].values - tmpList.loc[mask, 'fit_x'].values
+            tmpList.loc[mask, 'shift_y'] = tmpList.loc[
+                mask, 'detector_y'].values - tmpList.loc[mask, 'fit_y'].values
+
+            # MERGING SHIFTS INTO MAIN DATAFRAME
+            tmpList = tmpList.loc[tmpList['shift_x'].notnull(
+            ), ['wavelength', 'order', 'shift_x', 'shift_y']]
+            orderPixelTable = orderPixelTable.merge(tmpList, on=[
+                'wavelength', 'order'], how='outer')
+
+            # DROP ROWS WITH MISSING SHIFTS
+            orderPixelTable.dropna(axis='index', how='any', subset=[
+                'shift_x'], inplace=True)
+
+            # SHIFT DETECTOR LINE PIXEL POSITIONS BY SHIFTS
+            # UPDATE FILTERED VALUES
+            orderPixelTable.loc[
+                :, 'detector_x'] -= orderPixelTable.loc[:, 'shift_x']
+            orderPixelTable.loc[
+                :, 'detector_y'] -= orderPixelTable.loc[:, 'shift_y']
+
+            # DROP HELPER COLUMNS
+            orderPixelTable.drop(columns=['fit_x', 'fit_y',
+                                          'shift_x', 'shift_y'], inplace=True)
 
         self.log.debug('completed the ``get_predicted_line_list`` method')
-        return predictedLines
+        return orderPixelTable
 
     def detect_pinhole_arc_line(
             self,
-            x,
-            y,
-            windowHalf):
+            predictedLine):
         """*detect the observed position of an arc-line given the predicted pixel positions*
 
         **Key Arguments:**
-            - ``x`` -- the predicted x-positions
-            - ``y`` -- the predicted y-positions
-            - ``windowHalf`` -- half the width of the image stmap to cut and search (in pixels)
+            - ``predictedLine`` -- single predicted line coordinates from predicted line-list
 
         **Return:**
-            - ``observed_x`` -- the observed x-position of the line (None if not found)
-            - ``observed_y`` -- the observed y-position of the line (None if not found)
+            - ``predictedLine`` -- the line with the observed pixel coordinates appended (if detected, otherwise nan)
         """
         self.log.debug('starting the ``detect_pinhole_arc_line`` method')
 
         pinholeFrame = self.pinholeFrame
+        windowHalf = self.windowHalf
+        x = predictedLine['detector_x']
+        y = predictedLine['detector_y']
 
         # CLIP A STAMP FROM IMAGE AROUNDS PREDICTED POSITION
         xlow = int(np.max([x - windowHalf, 0]))
@@ -223,6 +266,7 @@ class create_dispersion_map(object):
 
         # USE DAOStarFinder TO FIND LINES WITH 2D GUASSIAN FITTING
         mean, median, std = sigma_clipped_stats(stamp, sigma=3.0)
+
         daofind = DAOStarFinder(
             fwhm=2.0, threshold=5. * std, roundlo=-3.0, roundhi=3.0, sharplo=-3.0, sharphi=3.0)
         sources = daofind(stamp - median)
@@ -249,12 +293,15 @@ class create_dispersion_map(object):
             #             ylow, marker='x', s=30)
             # plt.show()
         else:
-            observed_x = None
-            observed_y = None
+            observed_x = np.nan
+            observed_y = np.nan
         # plt.show()
 
+        predictedLine['observed_x'] = observed_x
+        predictedLine['observed_y'] = observed_y
+
         self.log.debug('completed the ``detect_pinhole_arc_line`` method')
-        return observed_x, observed_y
+        return predictedLine
 
     def write_map_to_file(
             self,
@@ -262,7 +309,7 @@ class create_dispersion_map(object):
             ycoeff,
             order_deg,
             wavelength_deg,
-            slit_deg=False):
+            slit_deg):
         """*write out the fitted polynomial solution coefficients to file*
 
         **Key Arguments:**
@@ -284,27 +331,41 @@ class create_dispersion_map(object):
         coeff_dict_x["axis"] = "x"
         coeff_dict_x["order-deg"] = order_deg
         coeff_dict_x["wavelength-deg"] = wavelength_deg
+        coeff_dict_x["slit-deg"] = slit_deg
         n_coeff = 0
         for i in range(0, order_deg + 1):
             for j in range(0, wavelength_deg + 1):
-                coeff_dict_x[f'c{i}{j}'] = xcoeff[n_coeff]
-                n_coeff += 1
+                for k in range(0, slit_deg + 1):
+                    coeff_dict_x[f'c{i}{j}{k}'] = xcoeff[n_coeff]
+                    n_coeff += 1
 
         # SORT Y COEFFICIENT OUTPUT TO WRITE TO FILE
         coeff_dict_y = {}
         coeff_dict_y["axis"] = "y"
         coeff_dict_y["order-deg"] = order_deg
         coeff_dict_y["wavelength-deg"] = wavelength_deg
+        coeff_dict_y["slit-deg"] = slit_deg
         n_coeff = 0
         for i in range(0, order_deg + 1):
             for j in range(0, wavelength_deg + 1):
-                coeff_dict_y[f'c{i}{j}'] = ycoeff[n_coeff]
-                n_coeff += 1
+                for k in range(0, slit_deg + 1):
+                    coeff_dict_y[f'c{i}{j}{k}'] = ycoeff[n_coeff]
+                    n_coeff += 1
 
         # DETERMINE WHERE TO WRITE THE FILE
         home = expanduser("~")
         outDir = self.settings["intermediate-data-root"].replace("~", home)
-        filePath = f"{outDir}/single_pinhole_{arm}_disp_map.csv"
+
+        filename = filenamer(
+            log=self.log,
+            frame=self.pinholeFrame,
+            settings=self.settings
+        )
+        if slit_deg == 0:
+            filename = filename.split("ARC")[0] + "DISP_MAP.csv"
+        else:
+            filename = filename.split("ARC")[0] + "2D_MAP.csv"
+        filePath = f"{outDir}/{filename}"
         dataSet = list_of_dictionaries(
             log=self.log,
             listOfDictionaries=[coeff_dict_x, coeff_dict_y]
@@ -316,22 +377,17 @@ class create_dispersion_map(object):
 
     def calculate_residuals(
             self,
-            observed_x,
-            observed_y,
-            wavelengths,
-            orders,
+            orderPixelTable,
             xcoeff,
             ycoeff,
             order_deg,
             wavelength_deg,
-            slit_deg=False):
-        """*calculate residuals of the polynomial fits against the observed line postions*
+            slit_deg):
+        """*calculate residuals of the polynomial fits against the observed line positions*
 
         **Key Arguments:**
-            - ``observed_x`` -- the measured x positions of the lines
-            - ``observed_x`` -- the measurd y positions of the lines
-            - ``wavelengths`` -- the wavelengths of the lines
-            - ``orders`` -- the orders of the lines
+
+            - ``orderPixelTable`` -- the predicted line list as a data frame
             - ``xcoeff`` -- the x-coefficients
             - ``ycoeff`` -- the y-coefficients
             - ``order_deg`` -- degree of the order fitting
@@ -348,47 +404,42 @@ class create_dispersion_map(object):
 
         arm = self.arm
 
-        # POLY FUNCTION NEEDS A TUPLE AS INPUT
-        order_wave = (orders, wavelengths)
-
+        # POLY FUNCTION NEEDS A DATAFRAME AS INPUT
         poly = chebyshev_order_wavelength_polynomials(
-            log=self.log, order_deg=order_deg, wavelength_deg=wavelength_deg).poly
+            log=self.log, order_deg=order_deg, wavelength_deg=wavelength_deg, slit_deg=slit_deg).poly
 
         # CALCULATE X & Y RESIDUALS BETWEEN OBSERVED LINE POSITIONS AND POLY
         # FITTED POSITIONS
-        xfit = poly(order_wave, *xcoeff)
-        yfit = poly(order_wave, *ycoeff)
-        residuals_x = np.asarray(xfit) - np.asarray(observed_x)
-        residuals_y = np.asarray(yfit) - np.asarray(observed_y)
+        orderPixelTable["fit_x"] = poly(orderPixelTable, *xcoeff)
+        orderPixelTable["fit_y"] = poly(orderPixelTable, *ycoeff)
+        orderPixelTable["residuals_x"] = orderPixelTable[
+            "fit_x"] - orderPixelTable["observed_x"]
+        orderPixelTable["residuals_y"] = orderPixelTable[
+            "fit_y"] - orderPixelTable["observed_y"]
 
         # CALCULATE COMBINED RESIDUALS AND STATS
-        combined_res = np.sqrt(np.square(residuals_x) + np.square(residuals_y))
-        combined_res_mean = np.mean(combined_res)
-        combined_res_std = np.std(combined_res)
-        combined_res_median = np.median(combined_res)
+        orderPixelTable["residuals_xy"] = np.sqrt(np.square(
+            orderPixelTable["residuals_x"]) + np.square(orderPixelTable["residuals_y"]))
+        combined_res_mean = np.mean(orderPixelTable["residuals_xy"])
+        combined_res_std = np.std(orderPixelTable["residuals_xy"])
+        combined_res_median = np.median(orderPixelTable["residuals_xy"])
 
         self.log.debug('completed the ``calculate_residuals`` method')
-        return combined_res, combined_res_mean, combined_res_std, combined_res_median, residuals_x, residuals_y, xfit, yfit
+        return combined_res_mean, combined_res_std, combined_res_median, orderPixelTable
 
     def fit_polynomials(
             self,
-            observed_x,
-            observed_y,
-            wavelengths,
-            orders,
-            order_deg,
+            orderPixelTable,
             wavelength_deg,
-            slit_deg=False):
+            order_deg,
+            slit_deg):
         """*iteratively fit the dispersion map polynomials to the data, clipping residuals with each iteration*
 
         **Key Arguments:**
-            - ``observed_x`` -- the measured x positions of the lines
-            - ``observed_x`` -- the measurd y positions of the lines
-            - ``wavelengths`` -- the wavelengths of the lines
-            - ``orders`` -- the orders of the lines
-            - ``order_deg`` -- degree of the order fitting
+            - ``orderPixelTable`` -- data frame containing order, wavelengths, slit positions and observed pixel positions
             - ``wavelength_deg`` -- degree of wavelength fitting
-            - ``slit_deg`` -- degree of the slit fitting (False for single pinhole)
+            - ``order_deg`` -- degree of the order fitting
+            - ``slit_deg`` -- degree of the slit fitting (0 for single pinhole)
 
         **Return:**
             - ``xcoeff`` -- the x-coefficients post clipping
@@ -398,53 +449,68 @@ class create_dispersion_map(object):
 
         arm = self.arm
 
+        if self.firstGuessMap:
+            recipe = "soxs-spatial-solution"
+        else:
+            recipe = "soxs-disp-solution"
+
         clippedCount = 1
 
         poly = chebyshev_order_wavelength_polynomials(
-            log=self.log, order_deg=order_deg, wavelength_deg=wavelength_deg).poly
+            log=self.log, order_deg=order_deg, wavelength_deg=wavelength_deg, slit_deg=slit_deg).poly
 
         clippingSigma = self.settings[
-            "soxs-disp-solution"]["poly-fitting-residual-clipping-sigma"]
-
+            recipe]["poly-fitting-residual-clipping-sigma"]
         clippingIterationLimit = self.settings[
-            "soxs-disp-solution"]["clipping-iteration-limit"]
+            recipe]["clipping-iteration-limit"]
 
         iteration = 0
         while clippedCount > 0 and iteration < clippingIterationLimit:
             iteration += 1
+            observed_x = orderPixelTable["observed_x"].to_numpy()
+            observed_y = orderPixelTable["observed_y"].to_numpy()
             # USE LEAST-SQUARED CURVE FIT TO FIT CHEBY POLYS
             # FIRST X
-            coeff = np.ones((order_deg + 1) * (wavelength_deg + 1))
-            xcoeff, pcov_x = curve_fit(poly, xdata=(
-                orders, wavelengths), ydata=observed_x, p0=coeff)
+            coeff = np.ones((order_deg + 1) *
+                            (wavelength_deg + 1) * (slit_deg + 1))
+            self.log.info("""curvefit x""" % locals())
+
+            xcoeff, pcov_x = curve_fit(
+                poly, xdata=orderPixelTable, ydata=observed_x, p0=coeff)
 
             # NOW Y
-            ycoeff, pcov_y = curve_fit(poly, xdata=(
-                orders, wavelengths), ydata=observed_y, p0=coeff)
+            self.log.info("""curvefit y""" % locals())
+            ycoeff, pcov_y = curve_fit(
+                poly, xdata=orderPixelTable, ydata=observed_y, p0=coeff)
 
-            residuals, mean_res, std_res, median_res, residuals_x, residuals_y, xfit, yfit = self.calculate_residuals(
-                observed_x=observed_x,
-                observed_y=observed_y,
-                wavelengths=wavelengths,
-                orders=orders,
+            self.log.info("""calculate_residuals""" % locals())
+            mean_res, std_res, median_res, orderPixelTable = self.calculate_residuals(
+                orderPixelTable=orderPixelTable,
                 xcoeff=xcoeff,
                 ycoeff=ycoeff,
                 order_deg=order_deg,
                 wavelength_deg=wavelength_deg,
-                slit_deg=False)
+                slit_deg=slit_deg)
 
             # SIGMA-CLIP THE DATA
+            self.log.info("""sigma_clip""" % locals())
             masked_residuals = sigma_clip(
-                residuals, sigma_lower=clippingSigma, sigma_upper=clippingSigma, maxiters=1, cenfunc='median', stdfunc=mad_std)
-
-            # MASK DATA ARRAYS WITH CLIPPED RESIDUAL MASK
-            startCount = len(observed_x)
-            a = [observed_x, observed_y, wavelengths, orders]
-            observed_x, observed_y, wavelengths, orders = [np.ma.compressed(np.ma.masked_array(
-                i, masked_residuals.mask)) for i in a]
-            clippedCount = startCount - len(observed_x)
+                orderPixelTable["residuals_xy"], sigma_lower=clippingSigma, sigma_upper=clippingSigma, maxiters=1, cenfunc='median', stdfunc=mad_std)
+            orderPixelTable["residuals_masked"] = masked_residuals.mask
+            # RETURN BREAKDOWN OF COLUMN VALUE COUNT
+            valCounts = orderPixelTable[
+                'residuals_masked'].value_counts(normalize=False)
+            if True in valCounts:
+                clippedCount = valCounts[True]
+            else:
+                clippedCount = 0
             print(f'{clippedCount} arc lines where clipped in this iteration of fitting a global dispersion map')
 
+            # REMOVE FILTERED ROWS FROM DATA FRAME
+            mask = (orderPixelTable['residuals_masked'] == True)
+            orderPixelTable.drop(index=orderPixelTable[
+                                 mask].index, inplace=True)
+
         # a = plt.figure(figsize=(40, 15))
         if arm == "UVB":
             fig = plt.figure(figsize=(6, 13.5), constrained_layout=True)
@@ -459,49 +525,69 @@ class create_dispersion_map(object):
         bottomright = fig.add_subplot(gs[4:, 2:])
 
         # ROTATE THE IMAGE FOR BETTER LAYOUT
-        rotatedImg = np.rot90(self.pinholeFrame.data, 1)
+        rotatedImg = np.flipud(np.rot90(self.pinholeFrame.data, 1))
         toprow.imshow(rotatedImg, vmin=10, vmax=50, cmap='gray', alpha=0.5)
         toprow.set_title(
             "observed arc-line positions (post-clipping)", fontsize=10)
-        x = np.ones(observed_x.shape) * \
-            self.pinholeFrame.data.shape[1] - observed_x
-        toprow.scatter(observed_y, x, marker='x', c='red', s=4)
+
+        x = np.ones(orderPixelTable.shape[0]) * \
+            self.pinholeFrame.data.shape[1] - orderPixelTable["observed_x"]
+        toprow.scatter(orderPixelTable["observed_y"],
+                       x, marker='x', c='red', s=4)
+
         # toprow.set_yticklabels([])
         # toprow.set_xticklabels([])
         toprow.set_ylabel("x-axis", fontsize=8)
         toprow.set_xlabel("y-axis", fontsize=8)
         toprow.tick_params(axis='both', which='major', labelsize=9)
+        toprow.invert_yaxis()
 
         midrow.imshow(rotatedImg, vmin=10, vmax=50, cmap='gray', alpha=0.5)
         midrow.set_title(
             "global dispersion solution", fontsize=10)
-        xfit = np.ones(len(xfit)) * \
-            self.pinholeFrame.data.shape[1] - xfit
-        midrow.scatter(yfit, xfit, marker='x', c='blue', s=4)
+
+        xfit = np.ones(orderPixelTable.shape[0]) * \
+            self.pinholeFrame.data.shape[1] - orderPixelTable["fit_x"]
+        midrow.scatter(orderPixelTable["fit_y"],
+                       xfit, marker='x', c='blue', s=4)
+
         # midrow.set_yticklabels([])
         # midrow.set_xticklabels([])
         midrow.set_ylabel("x-axis", fontsize=8)
         midrow.set_xlabel("y-axis", fontsize=8)
         midrow.tick_params(axis='both', which='major', labelsize=9)
+        midrow.invert_yaxis()
 
         # PLOT THE FINAL RESULTS:
         plt.subplots_adjust(top=0.92)
-        bottomleft.scatter(residuals_x, residuals_y, alpha=0.4)
+        bottomleft.scatter(orderPixelTable["residuals_x"], orderPixelTable[
+                           "residuals_y"], alpha=0.4)
         bottomleft.set_xlabel('x residual')
         bottomleft.set_ylabel('y residual')
         bottomleft.tick_params(axis='both', which='major', labelsize=9)
 
-        hist(residuals, bins='scott', ax=bottomright, histtype='stepfilled',
+        hist(orderPixelTable["residuals_xy"], bins='scott', ax=bottomright, histtype='stepfilled',
              alpha=0.7, density=True)
         bottomright.set_xlabel('xy residual')
         bottomright.tick_params(axis='both', which='major', labelsize=9)
         subtitle = f"mean res: {mean_res:2.2f} pix, res stdev: {std_res:2.2f}"
         fig.suptitle(f"residuals of global dispersion solution fitting - single pinhole\n{subtitle}", fontsize=12)
 
+        # GET FILENAME FOR THE RESIDUAL PLOT
+        filename = filenamer(
+            log=self.log,
+            frame=self.pinholeFrame,
+            settings=self.settings
+        )
+        if self.firstGuessMap:
+            filename = filename.split("ARC")[0] + "2D_MAP_RESIDUALS.pdf"
+        else:
+            filename = filename.split("ARC")[0] + "DISP_MAP_RESIDUALS.pdf"
+
         # plt.show()
         home = expanduser("~")
         outDir = self.settings["intermediate-data-root"].replace("~", home)
-        filePath = f"{outDir}/single_pinhole_{arm}_disp_map_residuals.pdf"
+        filePath = f"{outDir}/{filename}"
         plt.savefig(filePath)
 
         print(f'\nThe dispersion maps fitted against the observed arc-line positions with a mean residual of {mean_res:2.2f} pixels (stdev = {std_res:2.2f} pixles)')
