@@ -19,9 +19,8 @@ from soxspipe.commonutils import keyword_lookup
 from soxspipe.commonutils import detector_lookup
 from os.path import expanduser
 import numpy as np
-import unicodecsv as csv
-from soxspipe.commonutils.polynomials import chebyshev_order_wavelength_polynomials
 from soxspipe.commonutils.polynomials import chebyshev_xy_polynomial
+from soxspipe.commonutils.filenamer import filenamer
 import matplotlib.pyplot as plt
 from astropy.stats import mad_std
 from astropy.modeling import models, fitting
@@ -32,9 +31,172 @@ from astropy.stats import sigma_clip, mad_std
 from astropy.visualization import hist
 import collections
 from fundamentals.renderer import list_of_dictionaries
+from soxspipe.commonutils.dispersion_map_to_pixel_arrays import dispersion_map_to_pixel_arrays
+import pandas as pd
+from soxspipe.commonutils.toolkit import cut_image_slice
 
 
-class detect_continuum(object):
+class _base_detect(object):
+
+    def fit_polynomial(
+            self,
+            pixelList,
+            order,
+            xCol,
+            yCol):
+        """*iteratively fit the dispersion map polynomials to the data, clipping residuals with each iteration*
+
+        **Key Arguments:**
+            - ``pixelList`` -- data-frame group containing x,y pixel array
+            - ``order`` -- the order to fit
+            - ``xCol`` -- name of x-pixel column
+            - ``yCol`` -- name of y-pixel column
+
+        **Return:**
+            - ``coeffs`` -- the coefficients of the polynomial fit
+            - ``pixelList`` -- the pixel list but now with fits and residuals included
+        """
+        self.log.debug('starting the ``fit_polynomial`` method')
+
+        arm = self.arm
+
+        clippedCount = 1
+
+        poly = chebyshev_xy_polynomial(
+            log=self.log, deg=self.polyDeg).poly
+        clippingSigma = self.recipeSettings[
+            "poly-fitting-residual-clipping-sigma"]
+        clippingIterationLimit = self.recipeSettings[
+            "clipping-iteration-limit"]
+
+        iteration = 0
+        mask = (pixelList['order'] == order)
+        pixelListFiltered = pixelList.loc[mask]
+        while clippedCount > 0 and iteration < clippingIterationLimit:
+            pixelListFiltered = pixelList.loc[mask]
+
+            startCount = len(pixelListFiltered.index)
+            iteration += 1
+            # USE LEAST-SQUARED CURVE FIT TO FIT CHEBY POLY
+            coeff = np.ones((self.polyDeg + 1))
+            # NOTE X AND Y COLUMN ARE CORRECLY IN xdata AND ydata - WANT TO
+            # FIND X (UNKNOWN) WRT Y (KNOWNN)
+            try:
+                coeff, pcov_x = curve_fit(
+                    poly, xdata=pixelListFiltered[yCol].values, ydata=pixelListFiltered[xCol].values, p0=coeff)
+            except TypeError as e:
+                # REMOVE THIS ORDER FROM PIXEL LIST
+                pixelList.drop(index=pixelList[mask].index, inplace=True)
+                coeff = None
+                return coeff, pixelList
+            except Exception as e:
+                raise e
+
+            res, res_mean, res_std, res_median, xfit = self.calculate_residuals(
+                orderPixelTable=pixelListFiltered,
+                coeff=coeff,
+                xCol=xCol,
+                yCol=yCol)
+
+            pixelList.loc[mask, "x_fit_res"] = res
+            pixelList.loc[mask, "x_fit"] = xfit
+
+            # SIGMA-CLIP THE DATA
+            masked_residuals = sigma_clip(
+                res, sigma_lower=clippingSigma, sigma_upper=clippingSigma, maxiters=1, cenfunc='median', stdfunc=mad_std)
+            pixelList.loc[mask, "mask"] = masked_residuals.mask
+
+            # REMOVE FILTERED ROWS FROM DATA FRAME
+            removeMask = (pixelList["mask"] == True)
+            pixelList.drop(index=pixelList[removeMask].index, inplace=True)
+            pixelListFiltered = pixelList.loc[mask]
+            clippedCount = startCount - len(pixelListFiltered.index)
+
+            # MASK DATA ARRAYS WITH CLIPPED RESIDUAL MASK
+            print(f'{clippedCount} pixel positions where clipped in this iteration of fitting an order centre polynomial')
+
+        self.log.debug('completed the ``fit_polynomials`` method')
+        return coeff, pixelList
+
+    def calculate_residuals(
+            self,
+            orderPixelTable,
+            coeff,
+            xCol,
+            yCol):
+        """*calculate residuals of the polynomial fits against the observed line postions*
+
+        **Key Arguments:**
+            - ``orderPixelTable`` -- data-frame containing pixel list for given order
+            - ``coeff`` -- the coefficients of the fitted polynomial
+            - ``xCol`` -- name of x-pixel column
+            - ``yCol`` -- name of y-pixel column
+
+        **Return:**
+            - ``res`` -- x residuals
+            - ``mean`` -- the mean of the residuals
+            - ``std`` -- the stdev of the residuals
+            - ``median`` -- the median of the residuals
+            - ``xfit`` -- fitted x values
+        """
+        self.log.debug('starting the ``calculate_residuals`` method')
+
+        arm = self.arm
+
+        poly = chebyshev_xy_polynomial(
+            log=self.log, deg=self.polyDeg).poly
+
+        # CALCULATE RESIDUALS BETWEEN GAUSSIAN PEAK LINE POSITIONS AND POLY
+        # FITTED POSITIONS
+        xfit = poly(
+            orderPixelTable[yCol].values, *coeff)
+        res = xfit - orderPixelTable[xCol].values
+
+        # CALCULATE COMBINED RESIDUALS AND STATS
+        res_mean = np.mean(res)
+        res_std = np.std(res)
+        res_median = np.median(res)
+
+        self.log.debug('completed the ``calculate_residuals`` method')
+        return res, res_mean, res_std, res_median, xfit
+
+    def write_order_table_to_file(
+            self,
+            frame,
+            orderPolyTable):
+        """*write out the fitted polynomial solution coefficients to file*
+
+        **Key Arguments:**
+            - ``frame`` -- the calibration frame used to generate order location data
+            - ``orderPolyTable`` -- data-frames containing centre location coefficients (and possibly also order edge coeffs)
+
+        **Return:**
+            - ``order_table_path`` -- path to the order table file
+        """
+        self.log.debug('starting the ``write_order_table_to_file`` method')
+
+        arm = self.arm
+
+        # DETERMINE WHERE TO WRITE THE FILE
+        home = expanduser("~")
+        outDir = self.settings["intermediate-data-root"].replace("~", home)
+
+        filename = filenamer(
+            log=self.log,
+            frame=frame,
+            settings=self.settings
+        )
+        filename = filename.replace("MFLAT", "FLAT")
+        filename = filename.split("FLAT")[0] + "ORDER_LOCATIONS.csv"
+
+        order_table_path = f"{outDir}/{filename}"
+        orderPolyTable.to_csv(order_table_path, index=False)
+
+        self.log.debug('completed the ``write_order_table_to_file`` method')
+        return order_table_path
+
+
+class detect_continuum(_base_detect):
     """
     *find and fit the continuum in a pinhole flat frame with low-order polynomials. These polynominals are the central loctions of the orders*
 
@@ -42,7 +204,8 @@ class detect_continuum(object):
         - ``log`` -- logger
         - ``pinholeFlat`` -- calibrationed pinhole flat frame (CCDObject)
         - ``dispersion_map`` -- path to dispersion map csv file containing polynomial fits of the dispersion solution for the frame
-        - ``settings`` -- the settings dictionary
+        - ``settings`` -- the recipe settings dictionary
+        - ``recipeName`` -- the recipe name as given in the settings dictionary
 
     **Usage:**
 
@@ -54,11 +217,11 @@ class detect_continuum(object):
         log=log,
         pinholeFlat=pinholeFlat,
         dispersion_map=dispersion_map,
-        settings=settings
+        settings=settings,
+        recipeName="soxs-order-centre"
     )
     order_table_path = detector.get()
     ```
-
     """
 
     def __init__(
@@ -66,11 +229,16 @@ class detect_continuum(object):
             log,
             pinholeFlat,
             dispersion_map,
-            settings=False
+            settings=False,
+            recipeName=False
     ):
         self.log = log
         log.debug("instansiating a new 'detect_continuum' object")
         self.settings = settings
+        if recipeName:
+            self.recipeSettings = settings[recipeName]
+        else:
+            self.recipeSettings = False
         self.pinholeFlat = pinholeFlat
         self.dispersion_map = dispersion_map
 
@@ -89,8 +257,7 @@ class detect_continuum(object):
         ).get(self.arm)
 
         # DEG OF THE POLYNOMIALS TO FIT THE ORDER CENTRE LOCATIONS
-        self.polyDeg = self.settings[
-            "soxs-order-centre"]["poly-deg"]
+        self.polyDeg = self.recipeSettings["poly-deg"]
 
         return None
 
@@ -110,63 +277,84 @@ class detect_continuum(object):
 
         # CONVERT WAVELENGTH TO PIXEL POSTIONS AND RETURN ARRAY OF POSITIONS TO
         # SAMPLE THE TRACES
-        pixelArrays = self.create_pixel_arrays(
+        orderPixelTable = self.create_pixel_arrays(
             orderNums,
             waveLengthMin,
             waveLengthMax)
-
         # SLICE LENGTH TO SAMPLE TRACES IN THE CROSS-DISPERSION DIRECTION
-        sliceLength = self.settings[
-            "soxs-order-centre"]["slice-length"]
-        peakSigmaLimit = self.settings[
-            "soxs-order-centre"]["peak-sigma-limit"]
+        self.sliceLength = self.recipeSettings["slice-length"]
+        self.peakSigmaLimit = self.recipeSettings["peak-sigma-limit"]
+
+        # PREP LISTS WITH NAN VALUE IN CONT_X AND CONT_Y BEFORE FITTING
+        orderPixelTable['cont_x'] = np.nan
+        orderPixelTable['cont_y'] = np.nan
 
         # FOR EACH ORDER, FOR EACH PIXEL POSITION SAMPLE, FIT A 1D GAUSSIAN IN
         # CROSS-DISPERSION DIRECTTION. RETURN PEAK POSTIONS
-        guassianPixelArrays = {}
-        for order, xy_array in pixelArrays.items():
-            found_xy_array = []
-            for xy in xy_array:
-                found_xy = self.fit_1d_gaussian_to_slice(
-                    xy, sliceLength, peakSigmaLimit)
-                if found_xy:
-                    found_xy_array.append(found_xy)
-            guassianPixelArrays[order] = found_xy_array
+        orderPixelTable = orderPixelTable.apply(
+            self.fit_1d_gaussian_to_slice, axis=1)
+        allLines = len(orderPixelTable.index)
+        # DROP ROWS WITH NAN VALUES
+        orderPixelTable.dropna(axis='index', how='any',
+                               subset=['cont_x'], inplace=True)
+        foundLines = len(orderPixelTable.index)
+        percent = 100 * foundLines / allLines
+        print(f"{foundLines} out of {allLines} found ({percent:3.0f}%)")
 
-            percent = 100 * len(found_xy_array) / len(xy_array)
-            print(f"{len(found_xy_array)} out of {len(xy_array)} found ({percent:3.0f}%)")
+        # GET UNIQUE VALUES IN COLUMN
+        uniqueOrders = orderPixelTable['order'].unique()
 
-        orderLoctions = {}
-        allResiduals = []
-        allXfit = []
-        allXcoords = []
-        allYcoords = []
-        for order, pixelArray in guassianPixelArrays.items():
+        orderLocations = {}
+        orderPixelTable['x_fit'] = np.nan
+        orderPixelTable['x_fit_res'] = np.nan
+
+        for o in uniqueOrders:
             # ITERATIVELY FIT THE POLYNOMIAL SOLUTIONS TO THE DATA
-            coeff, residuals, xfit, xcoords, ycoords = self.fit_polynomial(
-                pixelArray=pixelArray
+            coeff, orderPixelTable = self.fit_polynomial(
+                pixelList=orderPixelTable,
+                order=o,
+                xCol="cont_x",
+                yCol="cont_y"
             )
-            allResiduals.extend(residuals)
-            allXfit.extend(xfit)
-            allXcoords.extend(xcoords)
-            allYcoords.extend(ycoords)
-            orderLoctions[order] = coeff
+            orderLocations[o] = coeff
 
-        self.plot_results(
-            allResiduals=allResiduals,
-            allXfit=allXfit,
-            allXcoords=allXcoords,
-            allYcoords=allYcoords,
-            orderLoctions=orderLoctions
+        # SORT CENTRE TRACE COEFFICIENT OUTPUT TO PANDAS DATAFRAME
+        columnsNames = ["order", "degy_cent", "ymin", "ymax"]
+        coeffColumns = [f'cent_c{i}' for i in range(0, self.polyDeg + 1)]
+        columnsNames.extend(coeffColumns)
+        myDict = {k: [] for k in columnsNames}
+        for k, v in orderLocations.items():
+            myDict["order"].append(k)
+            myDict["degy_cent"].append(self.polyDeg)
+            n_coeff = 0
+            for i in range(0, self.polyDeg + 1):
+                myDict[f'cent_c{i}'].append(v[n_coeff])
+                n_coeff += 1
+
+            myDict["ymin"].append(
+                np.min(orderPixelTable.loc[(orderPixelTable['order'] == k)]["fit_y"].values))
+            myDict["ymax"].append(
+                np.max(orderPixelTable.loc[(orderPixelTable['order'] == k)]["fit_y"].values))
+        orderPolyTable = pd.DataFrame(myDict)
+
+        # HERE IS THE LINE LIST IF NEEDED FOR QC
+        orderPixelTable.drop(columns=['mask'], inplace=True)
+
+        plotPath = self.plot_results(
+            orderPixelTable=orderPixelTable,
+            orderPolyTable=orderPolyTable
         )
 
-        mean_res = np.mean(np.abs(allResiduals))
-        std_res = np.std(np.abs(allResiduals))
+        mean_res = np.mean(np.abs(orderPixelTable['x_fit_res'].values))
+        std_res = np.std(np.abs(orderPixelTable['x_fit_res'].values))
 
-        print(f'\nThe order centre polynomial fitted against the observed 1D gaussian peak positions with a mean residual of {mean_res:2.2f} pixels (stdev = {std_res:2.2f} pixles)')
+        print(f'\nThe order centre polynomial fitted against the observed 1D gaussian peak positions with a mean residual of {mean_res:2.2f} pixels (stdev = {std_res:2.2f} pixels)')
 
         # WRITE OUT THE FITS TO THE ORDER CENTRE TABLE
-        order_table_path = self.write_order_table_to_file(orderLoctions)
+        order_table_path = self.write_order_table_to_file(
+            frame=self.pinholeFlat, orderPolyTable=orderPolyTable)
+
+        print(f'\nFind results of the order centre fitting here: {plotPath}')
 
         self.log.debug('completed the ``get`` method')
         return order_table_path
@@ -192,15 +380,16 @@ class detect_continuum(object):
             "calibration-data-root"].replace("~", home)
         spectralFormatFile = calibrationRootPath + \
             "/" + dp["spectral format table"]
-        spectralFormat = np.genfromtxt(
-            spectralFormatFile, delimiter=',', names=True)
 
-        # print(spectralFormat.dtype.names)
+        # READ CSV FILE TO PANDAS DATAFRAME
+        specFormatTable = pd.read_csv(
+            spectralFormatFile, index_col=False, na_values=['NA', 'MISSING'])
+        # # DATAFRAME INFO
 
         # EXTRACT REQUIRED PARAMETERS
-        orderNums = spectralFormat["ORDER"]
-        waveLengthMin = spectralFormat["WLMINFUL"]
-        waveLengthMax = spectralFormat["WLMAXFUL"]
+        orderNums = specFormatTable["ORDER"].values
+        waveLengthMin = specFormatTable["WLMINFUL"].values
+        waveLengthMax = specFormatTable["WLMAXFUL"].values
 
         self.log.debug('completed the ``read_spectral_format`` method')
         return orderNums, waveLengthMin, waveLengthMax
@@ -218,7 +407,7 @@ class detect_continuum(object):
             - ``waveLengthMax`` -- a list of the minimum wavelengths reached by each order
 
         **Return:**
-            - ``pixelArrays`` -- a dictionary of order based pixel arrays
+            - ``orderPixelTable`` -- a data-frame containing lines and associated pixel locations
         """
         self.log.debug('starting the ``create_pixel_arrays`` method')
 
@@ -227,79 +416,54 @@ class detect_continuum(object):
             "soxs-order-centre"]["order-sample-count"]
 
         # CREATE THE WAVELENGTH/ORDER ARRAYS TO BE CONVERTED TO PIXELS
-        orderArray = np.asarray([])
-        wavelengthArray = np.asarray([])
+        myDict = {
+            "order": np.asarray([]),
+            "wavelength": np.asarray([]),
+            "slit_position": np.asarray([])
+        }
         for o, wmin, wmax in zip(orderNums, waveLengthMin, waveLengthMax):
 
-            wavelengthArray = np.append(wavelengthArray, np.arange(
-                wmin, wmax, (wmax - wmin) / sampleCount))
-            orderArray = np.append(orderArray, np.ones(sampleCount) * o)
-        order_wave = (orderArray, wavelengthArray)
+            wlArray = np.arange(
+                wmin, wmax, (wmax - wmin) / sampleCount)
+            myDict["wavelength"] = np.append(myDict["wavelength"], wlArray)
+            myDict["order"] = np.append(
+                myDict["order"], np.ones(len(wlArray)) * o)
+            myDict["slit_position"] = np.append(
+                myDict["slit_position"], np.zeros(len(wlArray)))
 
-        # SETUP EMPTY PIXEL ARRAYS
-        pixelArrays = {f"o{k:0.0f}": [] for k in orderNums}
-
-        # READ THE FILE
-        home = expanduser("~")
-        dispersion_map = self.dispersion_map.replace("~", home)
-
-        with open(dispersion_map, 'rb') as csvFile:
-            csvReader = csv.DictReader(
-                csvFile, dialect='excel', delimiter=',', quotechar='"')
-            for row in csvReader:
-                axis = row["axis"]
-                order_deg = int(row["order-deg"])
-                wavelength_deg = int(row["wavelength-deg"])
-                coeff = [float(v) for k, v in row.items() if k not in [
-                    "axis", "order-deg", "wavelength-deg"]]
-                poly = chebyshev_order_wavelength_polynomials(
-                    log=self.log, order_deg=order_deg, wavelength_deg=wavelength_deg).poly
-
-                if axis == "x":
-                    xcoords = poly(order_wave, *coeff)
-                if axis == "y":
-                    ycoords = poly(order_wave, *coeff)
-        csvFile.close()
-
-        # for o, w, x, y in zip(orderArray, wavelengthArray, xcoords, ycoords):
-        #     print(o, w, x, y)
-
-        xcoords, ycoords, orderArray = zip(
-            *[(x, y, o) for x, y, o in zip(xcoords, ycoords, orderArray) if (x > 0 and y > 0)])
-
-        for o, x, y in zip(orderArray, xcoords, ycoords):
-            pixelArrays[f"o{o:0.0f}"].append((x, y))
+        orderPixelTable = pd.DataFrame(myDict)
+        orderPixelTable = dispersion_map_to_pixel_arrays(
+            log=self.log,
+            dispersionMapPath=self.dispersion_map,
+            orderPixelTable=orderPixelTable
+        )
 
         self.log.debug('completed the ``create_pixel_arrays`` method')
-        return pixelArrays
+        return orderPixelTable
 
     def fit_1d_gaussian_to_slice(
             self,
-            xy,
-            sliceLength,
-            peakSigmaLimit):
+            pixelPostion):
         """*cut a slice from the pinhole flat along the cross-dispersion direction centred on pixel position, fit 1D gaussian and return the peak pixel position*
 
         **Key Arguments:**
-            - ``xy`` -- the x,y pixel coordinate (tuple)
-            - ``sliceLength`` -- length of the slice to cut
-            - ``peakSigmaLimit`` -- number of simga above median. If peak is not above this level slice will be rejected
+            - ``pixelPostion`` -- the x,y pixel coordinate from orderPixelTable data-frame (series)
 
         **Return:**
-            - ``fit_xy`` -- gaussian fit peak xy position
+            - ``pixelPostion`` -- now including gaussian fit peak xy position
         """
         self.log.debug('starting the ``fit_1d_gaussian_to_slice`` method')
 
         # CLIP OUT A SLICE TO INSPECT CENTRED AT POSITION
-        halfSlice = sliceLength / 2
-        x_fit = xy[0]
-        y_fit = xy[1]
+        halfSlice = self.sliceLength / 2
 
-        try:
-            slice = self.pinholeFlat.data[int(y_fit), max(
-                0, int(x_fit - halfSlice)):min(2048, int(x_fit + halfSlice))]
-        except:
-            return None
+        slice = cut_image_slice(log=self.log, frame=self.pinholeFlat,
+                                width=1, length=self.sliceLength, x=pixelPostion["fit_x"], y=pixelPostion["fit_y"], median=True, plot=False)
+
+        if slice is None:
+            pixelPostion["cont_x"] = np.nan
+            pixelPostion["cont_y"] = np.nan
+            return pixelPostion
 
         # CHECK THE SLICE POINTS IF NEEDED
         if 1 == 0:
@@ -314,8 +478,14 @@ class detect_continuum(object):
         # CONTAINING JUST NOISE
         median_r = np.median(slice)
         std_r = mad_std(slice)
+
+        if not median_r:
+            pixelPostion["cont_x"] = np.nan
+            pixelPostion["cont_y"] = np.nan
+            return pixelPostion
+
         peaks, _ = find_peaks(slice, height=median_r +
-                              peakSigmaLimit * std_r, width=1)
+                              self.peakSigmaLimit * std_r, width=1)
 
         # CHECK PEAK HAS BEEN FOUND
         if peaks is None or len(peaks) <= 0:
@@ -327,7 +497,9 @@ class detect_continuum(object):
                 plt.xlabel('Position')
                 plt.ylabel('Flux')
                 plt.show()
-            return None
+            pixelPostion["cont_x"] = np.nan
+            pixelPostion["cont_y"] = np.nan
+            return pixelPostion
 
         # FIT THE DATA USING A 1D GAUSSIAN - USING astropy.modeling
         # CENTRE THE GAUSSIAN ON THE PEAK
@@ -338,8 +510,9 @@ class detect_continuum(object):
 
         # NOW FIT
         g = fit_g(g_init, np.arange(0, len(slice)), slice)
-        xpeak = g.mean + max(0, int(x_fit - halfSlice))
-        ypeak = y_fit
+        pixelPostion["cont_x"] = g.mean + \
+            max(0, int(pixelPostion["fit_x"] - halfSlice))
+        pixelPostion["cont_y"] = pixelPostion["fit_y"]
 
         # PRINT A FEW PLOTS IF NEEDED - GUASSIAN FIT OVERLAYED
         if 1 == 0 and random() < 0.02:
@@ -353,123 +526,17 @@ class detect_continuum(object):
             plt.show()
 
         self.log.debug('completed the ``fit_1d_gaussian_to_slice`` method')
-        return (xpeak, ypeak)
-
-    def fit_polynomial(
-            self,
-            pixelArray):
-        """*iteratively fit the dispersion map polynomials to the data, clipping residuals with each iteration*
-
-        **Key Arguments:**
-            - ``pixelArray`` -- the array of x,y pixels of measured 1D guassian peak positions
-
-        **Return:**
-            - ``coeffs`` -- the coefficients of the polynomial fit
-            - ``residuals`` -- the residuals of the fit compared to original guassian peak positions
-            - ``xfit`` -- the polynomial fits x-positions
-            - ``xcoords`` -- the clean guassian peak x-coordinate list (post-clipping)
-            - ``ycoords`` -- the clean guassian peak x-coordinate list (post-clipping)
-        """
-        self.log.debug('starting the ``fit_polynomial`` method')
-
-        arm = self.arm
-
-        clippedCount = 1
-
-        poly = chebyshev_xy_polynomial(
-            log=self.log, deg=self.polyDeg).poly
-
-        clippingSigma = self.settings[
-            "soxs-order-centre"]["poly-fitting-residual-clipping-sigma"]
-
-        clippingIterationLimit = self.settings[
-            "soxs-order-centre"]["clipping-iteration-limit"]
-
-        xcoords = [v[0] for v in pixelArray]
-        ycoords = [v[1] for v in pixelArray]
-
-        iteration = 0
-        while clippedCount > 0 and iteration < clippingIterationLimit:
-            iteration += 1
-            # USE LEAST-SQUARED CURVE FIT TO FIT CHEBY POLY
-            coeff = np.ones((self.polyDeg + 1))
-            coeff, pcov_x = curve_fit(
-                poly, xdata=ycoords, ydata=xcoords, p0=coeff)
-
-            residuals, mean_res, std_res, median_res, xfit = self.calculate_residuals(
-                xcoords=xcoords,
-                ycoords=ycoords,
-                coeff=coeff)
-
-            # SIGMA-CLIP THE DATA
-            masked_residuals = sigma_clip(
-                residuals, sigma_lower=clippingSigma, sigma_upper=clippingSigma, maxiters=1, cenfunc='median', stdfunc=mad_std)
-
-            # MASK DATA ARRAYS WITH CLIPPED RESIDUAL MASK
-            startCount = len(xcoords)
-            a = [xcoords, ycoords, residuals]
-            xcoords, ycoords, residuals = [np.ma.compressed(np.ma.masked_array(
-                i, masked_residuals.mask)) for i in a]
-            clippedCount = startCount - len(xcoords)
-            print(f'{clippedCount} pixel positions where clipped in this iteration of fitting an order centre polynomial')
-
-        self.log.debug('completed the ``fit_polynomials`` method')
-        return coeff, residuals, xfit, xcoords, ycoords
-
-    def calculate_residuals(
-            self,
-            xcoords,
-            ycoords,
-            coeff):
-        """*calculate residuals of the polynomial fits against the observed line postions*
-
-        **Key Arguments:**
-            - ``xcoords`` -- the measured x positions of the gaussian peaks
-            - ``ycoords`` -- the measurd y positions of the gaussian peaks
-            - ``coeff`` -- the coefficients of the fitted polynomial
-
-        **Return:**
-            - ``residuals`` -- x residuals
-            - ``mean`` -- the mean of the residuals
-            - ``std`` -- the stdev of the residuals
-            - ``median`` -- the median of the residuals
-        """
-        self.log.debug('starting the ``calculate_residuals`` method')
-
-        arm = self.arm
-
-        poly = chebyshev_xy_polynomial(
-            log=self.log, deg=self.polyDeg).poly
-
-        # CALCULATE RESIDUALS BETWEEN GAUSSIAN PEAK LINE POSITIONS AND POLY
-        # FITTED POSITIONS
-        xfit = poly(
-            ycoords, *coeff)
-        res_x = np.asarray(xfit) - np.asarray(xcoords)
-
-        # CALCULATE COMBINED RESIDUALS AND STATS
-        res_mean = np.mean(res_x)
-        res_std = np.std(res_x)
-        res_median = np.median(res_x)
-
-        self.log.debug('completed the ``calculate_residuals`` method')
-        return res_x, res_mean, res_std, res_median, xfit
+        return pixelPostion
 
     def plot_results(
             self,
-            allResiduals,
-            allXfit,
-            allXcoords,
-            allYcoords,
-            orderLoctions):
+            orderPixelTable,
+            orderPolyTable):
         """*generate a plot of the polynomial fits and residuals*
 
         **Key Arguments:**
-            - ``allResiduals`` -- list of all residuals
-            - ``allXfit`` -- list of all fitted x-positions 
-            - ``allXcoords`` -- cleaned list of all guassian x-pixel positions
-            - ``allYcoords`` -- cleaned list of all guassian y-pixel positions
-            - ``orderLoctions`` -- dictionary of order-location polynomial coeff
+            - ``orderPixelTable`` -- the pixel table with residuals of fits
+            - ``orderPolyTable`` -- data-frame of order-location polynomial coeff
 
         **Return:**
             - ``filePath`` -- path to the plot pdf
@@ -496,9 +563,10 @@ class detect_continuum(object):
         toprow.imshow(rotatedImg, vmin=10, vmax=50, cmap='gray', alpha=0.5)
         toprow.set_title(
             "1D guassian peak positions (post-clipping)", fontsize=10)
-        x = np.ones(len(allXcoords)) * \
-            self.pinholeFlat.data.shape[1] - allXcoords
-        toprow.scatter(allYcoords, x, marker='x', c='red', s=4)
+        x = np.ones(len(orderPixelTable.index)) * \
+            self.pinholeFlat.data.shape[1] - orderPixelTable['cont_x'].values
+        toprow.scatter(orderPixelTable[
+                       'cont_y'].values, x, marker='x', c='red', s=4)
         # toprow.set_yticklabels([])
         # toprow.set_xticklabels([])
         toprow.set_ylabel("x-axis", fontsize=8)
@@ -511,7 +579,11 @@ class detect_continuum(object):
         ylinelist = np.arange(0, self.pinholeFlat.data.shape[0], 3)
         poly = chebyshev_xy_polynomial(
             log=self.log, deg=self.polyDeg).poly
-        for o, coeff in orderLoctions.items():
+
+        # ONLY DO THIS FOR SMALL DATAFRAMES - THIS IS AN ANTIPATTERN
+        for index, row in orderPolyTable.iterrows():
+            o = row["order"]
+            coeff = [float(v) for k, v in row.items() if "cent_" in k]
             xfit = poly(ylinelist, *coeff)
             xfit = np.ones(len(xfit)) * \
                 self.pinholeFlat.data.shape[1] - xfit
@@ -530,73 +602,42 @@ class detect_continuum(object):
 
         # PLOT THE FINAL RESULTS:
         plt.subplots_adjust(top=0.92)
-        bottomleft.scatter(allXcoords, allResiduals, alpha=0.2, s=1)
+        bottomleft.scatter(orderPixelTable['cont_x'].values, orderPixelTable[
+                           'x_fit_res'].values, alpha=0.2, s=1)
         bottomleft.set_xlabel('x pixel position')
         bottomleft.set_ylabel('x residual')
         bottomleft.tick_params(axis='both', which='major', labelsize=9)
 
         # PLOT THE FINAL RESULTS:
         plt.subplots_adjust(top=0.92)
-        bottomright.scatter(allYcoords, allResiduals, alpha=0.2, s=1)
+        bottomright.scatter(orderPixelTable['cont_y'].values, orderPixelTable[
+                            'x_fit_res'].values, alpha=0.2, s=1)
         bottomright.set_xlabel('y pixel position')
         bottomright.tick_params(axis='both', which='major', labelsize=9)
         # bottomright.set_ylabel('x residual')
         bottomright.set_yticklabels([])
 
-        mean_res = np.mean(np.abs(allResiduals))
-        std_res = np.std(np.abs(allResiduals))
+        mean_res = np.mean(np.abs(orderPixelTable['x_fit_res'].values))
+        std_res = np.std(np.abs(orderPixelTable['x_fit_res'].values))
 
         subtitle = f"mean res: {mean_res:2.2f} pix, res stdev: {std_res:2.2f}"
         fig.suptitle(f"traces of order-centre locations - pinhole flat-frame\n{subtitle}", fontsize=12)
 
         # plt.show()
+        filename = filenamer(
+            log=self.log,
+            frame=self.pinholeFlat,
+            settings=self.settings
+        )
+        filename = filename.split("FLAT")[0] + "ORDER_CENTRES_residuals.pdf"
+
         home = expanduser("~")
         outDir = self.settings["intermediate-data-root"].replace("~", home)
-        filePath = f"{outDir}/pinhole_flat_{arm}_order_location_residuals.pdf"
+        filePath = f"{outDir}/{filename}"
         plt.savefig(filePath)
 
         self.log.debug('completed the ``plot_results`` method')
         return filePath
 
-    def write_order_table_to_file(
-            self,
-            orderLoctions):
-        """*write out the fitted polynomial solution coefficients to file*
-
-        **Key Arguments:**
-            - ``orderLoctions`` -- dictionary of the order coefficients
-
-        **Return:**
-            - ``order_table_path`` -- path to the order table file
-        """
-        self.log.debug('starting the ``write_order_table_to_file`` method')
-
-        arm = self.arm
-
-        # SORT COEFFICIENT OUTPUT TO WRITE TO FILE
-        listOfDictionaries = []
-        for k, v in orderLoctions.items():
-            orderDict = collections.OrderedDict(sorted({}.items()))
-            orderDict["order"] = int(k.replace("o", ""))
-            orderDict["degy"] = self.polyDeg
-            n_coeff = 0
-            for i in range(0, self.polyDeg + 1):
-                orderDict[f'CENT_c{i}'] = v[n_coeff]
-                n_coeff += 1
-            listOfDictionaries.append(orderDict)
-
-        # DETERMINE WHERE TO WRITE THE FILE
-        home = expanduser("~")
-        outDir = self.settings["intermediate-data-root"].replace("~", home)
-        order_table_path = f"{outDir}/order_centre_{arm}_locations.csv"
-        dataSet = list_of_dictionaries(
-            log=self.log,
-            listOfDictionaries=listOfDictionaries
-        )
-        csvData = dataSet.csv(filepath=order_table_path)
-
-        self.log.debug('completed the ``write_order_table_to_file`` method')
-        return order_table_path
-
     # use the tab-trigger below for new method
     # xt-class-method
