@@ -6,3 +6,6 @@ from .soxs_mbias import soxs_mbias
 from .soxs_mdark import soxs_mdark
 from .soxs_disp_solution import soxs_disp_solution
 from .soxs_order_centres import soxs_order_centres
+from .soxs_mflat import soxs_mflat
+from .soxs_spatial_solution import soxs_spatial_solution
+from .soxs_straighten import soxs_straighten
