@@ -26,6 +26,7 @@ from soxspipe.commonutils import set_of_files
 from soxspipe.commonutils import keyword_lookup
 from soxspipe.commonutils import detector_lookup
 from datetime import datetime
+from soxspipe.commonutils import filenamer
 import shutil
 
 
@@ -104,9 +105,9 @@ class _base_recipe_(object):
             frame = CCDData.read(frame, hdu=0, unit=u.adu, hdu_uncertainty='ERRS',
                                  hdu_mask='QUAL', hdu_flags='FLAGS', key_uncertainty_type='UTYPE')
 
-        # CHECK THE NUMBER OF EXTENSIONS IS ONLY 1 AND "SOXSPIPE PRE" DOES NOT
+        # CHECK THE NUMBER OF EXTENSIONS IS ONLY 1 AND "SXSPRE" DOES NOT
         # EXIST. i.e. THIS IS A RAW UNTOUCHED FRAME
-        if len(frame.to_hdu()) > 1 or "SOXSPIPE PRE" in frame.header:
+        if len(frame.to_hdu()) > 1 or "SXSPRE" in frame.header:
             return filepath
 
         # MANIPULATE XSH DATA
@@ -179,7 +180,7 @@ class _base_recipe_(object):
 
         # INJECT THE PRE KEYWORD
         utcnow = datetime.utcnow()
-        frame.header["SOXSPIPE PRE"] = (utcnow.strftime(
+        frame.header["SXSPRE"] = (utcnow.strftime(
             "%Y-%m-%dT%H:%M:%S.%f"), "UTC timestamp")
 
         # RECURSIVELY CREATE MISSING DIRECTORIES
@@ -194,7 +195,12 @@ class _base_recipe_(object):
             filenameNoExtension + "_pre" + extension
 
         # SAVE TO DISK
-        self._write(frame, filePath)
+        self._write(
+            frame=frame,
+            filedir=outDir,
+            filename=filenameNoExtension + "_pre" + extension,
+            overwrite=True
+        )
 
         self.log.debug('completed the ``_prepare_single_frame`` method')
         return filePath
@@ -320,7 +326,6 @@ class _base_recipe_(object):
 
         if cdelt1[0] and cdelt2[0]:
             self.detectorParams["binning"] = [int(cdelt2[0]), int(cdelt1[0])]
-            print(self.detectorParams["binning"])
 
         # MIXED READOUT SPEEDS IS BAD
         readSpeed = self.inputFrames.values(
@@ -452,13 +457,15 @@ class _base_recipe_(object):
     def _write(
             self,
             frame,
-            filepath,
+            filedir,
+            filename=False,
             overwrite=True):
         """*write frame to disk at the specified location*
 
         **Key Arguments:**
             - ``frame`` -- the frame to save to disk (CCDData object)
-            - ``filepath`` -- the location to save the frame
+            - ``filedir`` -- the location to save the frame
+            - ``filename`` -- the filename to save the file as. Default: **False** (standardised filename generated in code)
             - ``overwrite`` -- if a file exists at the filepath then choose to overwrite the file. Default: True
 
         **Usage:**
@@ -471,6 +478,16 @@ class _base_recipe_(object):
         """
         self.log.debug('starting the ``write`` method')
 
+        if not filename:
+
+            filename = filenamer(
+                log=self.log,
+                frame=frame,
+                settings=self.settings
+            )
+
+        filepath = filedir + "/" + filename
+
         HDUList = frame.to_hdu(
             hdu_mask='QUAL', hdu_uncertainty='ERRS', hdu_flags=None)
         HDUList[0].name = "FLUX"
@@ -478,7 +495,7 @@ class _base_recipe_(object):
                         overwrite=overwrite, checksum=True)
 
         self.log.debug('completed the ``write`` method')
-        return None
+        return filepath
 
     def clip_and_stack(
             self,
@@ -487,7 +504,7 @@ class _base_recipe_(object):
         """*mean combine input frames after sigma-clipping outlying pixels using a median value with median absolute deviation (mad) as the deviation function*
 
         **Key Arguments:**
-            - ``frames`` -- an ImageFileCollection of the framers to stack
+            - ``frames`` -- an ImageFileCollection of the frames to stack or a list of CCDData objects
             - ``recipe`` -- the name of recipe needed to read the correct settings from the yaml files
 
         **Return:**
@@ -530,8 +547,17 @@ class _base_recipe_(object):
 
         # LIST OF CCDDATA OBJECTS NEEDED BY COMBINER OBJECT
         # ccds = [c for c in self.inputFrames.ccds()]
-        ccds = [c for c in self.inputFrames.ccds(ccd_kwargs={"hdu_uncertainty": 'ERRS',
-                                                             "hdu_mask": 'QUAL', "hdu_flags": 'FLAGS', "key_uncertainty_type": 'UTYPE'})]
+        if not isinstance(frames, list):
+            ccds = [c for c in frames.ccds(ccd_kwargs={"hdu_uncertainty": 'ERRS',
+                                                       "hdu_mask": 'QUAL', "hdu_flags": 'FLAGS', "key_uncertainty_type": 'UTYPE'})]
+        else:
+            ccds = frames
+
+        # COMBINE MASKS AND THEN RESET
+        combinedMask = ccds[0].mask
+        for c in ccds:
+            combinedMask = c.mask | combinedMask
+            c.mask[:, :] = False
 
         # COMBINER OBJECT WILL FIRST GENERATE MASKS FOR INDIVIDUAL IMAGES VIA
         # CLIPPING AND THEN COMBINE THE IMAGES WITH THE METHOD SELECTED. PIXEL
@@ -540,8 +566,8 @@ class _base_recipe_(object):
 
         print(f"\n# SIGMA-CLIPPING PIXEL WITH OUTLYING VALUES IN INDIVIDUAL {imageType} FRAMES")
         # PRINT SOME INFO FOR USER
-        badCount = ccds[0].mask.sum()
-        totalPixels = np.size(ccds[0].mask)
+        badCount = combinedMask.sum()
+        totalPixels = np.size(combinedMask)
         percent = (float(badCount) / float(totalPixels)) * 100.
         print(f"The basic bad-pixel mask for the {arm} detector {imageType} frames contains {badCount} pixels ({percent:0.2}% of all pixels)")
 
@@ -568,9 +594,11 @@ class _base_recipe_(object):
         print("\n# MEAN COMBINING FRAMES - WITH UPDATED BAD-PIXEL MASKS")
         combined_frame = combiner.average_combine()
 
+        # RECOMBINE THE COMBINED MASK FROM ABOVE
+        combined_frame.mask = combined_frame.mask | combinedMask
+
         # MASSIVE FUDGE - NEED TO CORRECTLY WRITE THE HEADER FOR COMBINED
         # IMAGES
-
         combined_frame.header = ccds[0].header
         try:
             combined_frame.wcs = ccds[0].wcs
@@ -625,6 +653,11 @@ class _base_recipe_(object):
         kw = self.kw
         dp = self.detectorParams
 
+        if master_bias == None:
+            master_bias = False
+        if dark == None:
+            dark = False
+
         # VERIFY DATA IS IN ORDER
         if master_bias == False and dark == False:
             raise TypeError(
@@ -637,7 +670,8 @@ class _base_recipe_(object):
                 "CODE NEEDS WRITTEN HERE TO SCALE DARK FRAME TO EXPOSURE TIME OF SCIENCE/CALIBRATION FRAME")
 
         # DARK WITH MATCHING EXPOSURE TIME
-        if dark != False and dark.header[kw("EXPTIME")] == inputFrame.header[kw("EXPTIME")]:
+        tolerence = 0.5
+        if dark != False and (int(dark.header[kw("EXPTIME")]) < int(inputFrame.header[kw("EXPTIME")]) + tolerence) and (int(dark.header[kw("EXPTIME")]) > int(inputFrame.header[kw("EXPTIME")]) - tolerence):
             calibration_subtracted_frame = inputFrame.subtract(dark)
             calibration_subtracted_frame.header = inputFrame.header
             try:
