@@ -44,6 +44,8 @@ class soxs_mdark(_base_recipe_):
     )..produce_product()
     ```
 
+    ---
+
     ```eval_rst
     .. todo::
 
@@ -63,6 +65,7 @@ class soxs_mdark(_base_recipe_):
         self.log = log
         log.debug("instansiating a new 'soxs_mdark' object")
         self.settings = settings
+        self.recipeSettings = settings["soxs-mdark"]
         self.inputFrames = inputFrames
         # xt-self-arg-tmpx
 
@@ -83,7 +86,7 @@ class soxs_mdark(_base_recipe_):
         sys.stdout.write("\x1b[1A\x1b[2K")
         print("# VERIFYING INPUT FRAMES - ALL GOOD")
 
-        print("\n# RAW INPUT DARK FRAMES - SUMMARY")
+        print("\n# RAW INPUT FRAMES - SUMMARY")
         # SORT IMAGE COLLECTION
         self.inputFrames.sort(['mjd-obs'])
         print(self.inputFrames.summary, "\n")
@@ -97,7 +100,7 @@ class soxs_mdark(_base_recipe_):
 
     def verify_input_frames(
             self):
-        """*verify the input frame match those required by the soxs_mdark recipe*
+        """*verify input frame match those required by the soxs_mdark recipe*
 
         If the fits files conform to required input for the recipe everything will pass silently, otherwise an exception shall be raised.
         """
@@ -138,10 +141,10 @@ class soxs_mdark(_base_recipe_):
 
     def produce_product(
             self):
-        """*The code to generate the product of the soxs_mdark recipe*
+        """*generate a master dark frame*
 
         **Return:**
-            - ``productPath`` -- the path to the final product
+            - ``productPath`` -- the path to master dark frame
         """
         self.log.debug('starting the ``produce_product`` method')
 
@@ -152,13 +155,13 @@ class soxs_mdark(_base_recipe_):
         combined_bias_mean = self.clip_and_stack(
             frames=self.inputFrames, recipe="soxs_mdark")
 
-        x = int(dp["binning"][1])
-        y = int(dp["binning"][0])
-        productPath = self.intermediateRootPath + \
-            "/master_dark_%(arm)s_%(x)sx%(y)s.fits" % locals()
-
         # WRITE TO DISK
-        self._write(combined_bias_mean, productPath, overwrite=True)
+        productPath = self._write(
+            frame=combined_bias_mean,
+            filedir=self.intermediateRootPath,
+            filename=False,
+            overwrite=True
+        )
         self.clean_up()
 
         self.log.debug('completed the ``produce_product`` method')
